# start with largest first, to make conversion easy
roman_to_int = {
    'M' : 1000,
    'CM' : 900,
    'D' : 500,
    'CD' : 400,
    'C' : 100,
    'XC' : 90,
    'L' : 50,
    'XL' : 40,
    'X' : 10,
    'IX' : 9,
    'V' : 5,
    'IV' : 4,
    'I' : 1
}

def from_roman(roman):
    """Convert a roman number to integer"""
    num = 0
    while roman:
        try:
            # if one of the 2-char forms matches, use that
            two_chars, rest = roman[:2], roman[2:]
            num += roman_to_int[two_chars]
        except KeyError:
            # otherwise translate one char
            one_char, rest = roman[:1], roman[1:]
            num += roman_to_int[one_char]

        roman = rest
    return num

def to_roman(num):
    """Convert an integer to the shortest roman number"""
    roman = ''
    while num:
        for r, i in roman_to_int.items():
            # find the first roman letter that fits
            # then go back to the while-loop
            if num >= i:
                num -= i
                roman += r
                break
    return roman

def simplify(roman):
    """Convert a roman number to shortest form"""
    return to_roman(from_roman(roman))

def solve_euler_089(filename):
    """
    How many characters are saved by converting all roman numbers
    in 'filename' to shortest form?
    """
    sum_diffs = 0
    with open(filename) as f:
        for line in f:
            roman = line.strip()
            len_before = len(roman)
            len_after = len(simplify(roman))
            sum_diffs += len_before - len_after
    return sum_diffs


if __name__ == "__main__":
    solution = solve_euler_089('roman.txt')
    print(solution)
    if solution == 743:
        print('OK!')
